<?php

namespace PB\WithdrawalQueue\Admin\Controller;

use PB\WithdrawalQueue\Addon;
use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;

class RequisiteField extends \XF\Admin\Controller\AbstractField
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertAdminPermission('pb_wq_manage');
	}

	protected function getClassIdentifier()
	{
		return Addon::shortName('RequisiteField');
	}

	protected function getLinkPrefix()
	{
		return 'withdraw/requisite-fields';
	}

	protected function getTemplatePrefix()
	{
		return Addon::prefix('requisite_field');
	}

	protected function fieldAddEditResponse(\XF\Entity\AbstractField $field)
	{
		$reply = parent::fieldAddEditResponse($field);

		if ($reply instanceof \XF\Mvc\Reply\View)
		{
			/** @var \PB\WithdrawalQueue\Repository\PaymentSystem $paymentSystemRepo */
			$paymentSystemRepo = $this->repository(Addon::shortName('PaymentSystem'));
			$paymentSystems = $paymentSystemRepo->findPaymentSystems();

			/** @var \XF\Mvc\Entity\ArrayCollection $requisiteFieldAssociations */
			$requisiteFieldAssociations = $field->getRelationOrDefault('PaymentSystemFields', false);

			$reply->setParams([
				'paymentSystems' => $paymentSystems,
				'paymentSystemIds' => $requisiteFieldAssociations->pluckNamed('system_id')
			]);
		}

		return $reply;
	}

	protected function saveAdditionalData(FormAction $form, \XF\Entity\AbstractField $field)
	{
		$nodeIds = $this->filter('system_ids', 'array-uint');

		/** @var \PB\WithdrawalQueue\Entity\RequisiteField $field */
		$form->complete(function() use($field, $nodeIds)
		{
			/** @var \PB\WithdrawalQueue\Repository\PaymentSystemField $repo */
			$repo = $this->repository(Addon::shortName('PaymentSystemField'));
			$repo->updateFieldAssociations($field, $nodeIds);
		});

		return $form;
	}
}