<?php

namespace PB\WithdrawalQueue\Api\Controller;

use PB\WithdrawalQueue\Addon;
use XF\Mvc\Entity\Entity;
use XF\Mvc\ParameterBag;

class PaymentSystem extends \XF\Api\Controller\AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertApiScopeByRequestMethod('pb_wq_payment_system');
	}

	/**
	 * @api-desc Gets information about the specified payment system.
	 *
	 * @api-out PB_WithdrawalQueue_PaymentSystem $paymentSystem
	 */
	public function actionGet(ParameterBag $params)
	{
		$paymentSystem = $this->assertPaymentSystemExists($params->system_id);

		$result = [
			'paymentSystem' => $paymentSystem->toApiResult(Entity::VERBOSITY_VERBOSE)
		];

		return $this->apiResult($result);
	}


	/**
	 * @api-desc Deletes the specified payment system.
	 *
	 * @api-out true $success
	 *
	 * @param ParameterBag $params
	 * @return \XF\Api\Mvc\Reply\ApiResult|\XF\Mvc\Reply\AbstractReply|\XF\Mvc\Reply\Error
	 * @throws \XF\Mvc\Reply\Exception
	 * @throws \XF\PrintableException
	 */
	public function actionDelete(ParameterBag $params)
	{
		$paymentSystem = $this->assertPaymentSystemExists($params->system_id);
		if (!$paymentSystem->preDelete())
		{
			return $this->error($paymentSystem->getErrors());
		}

		$paymentSystem->delete();

		return $this->apiSuccess();
	}

	/**
	 * @param $id
	 * @param null $with
	 * @return \XF\Mvc\Entity\Entity|\PB\WithdrawalQueue\Entity\PaymentSystem
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertPaymentSystemExists($id, $with = null)
	{
		return $this->assertRecordExists(
			Addon::shortName('PaymentSystem'),
			$id,
			$with,
			Addon::phrase('requested_payment_system_not_found')
		);
	}
}