<?php

namespace PB\WithdrawalQueue\Api\Controller;

use PB\WithdrawalQueue\Addon;
use PB\WithdrawalQueue\Entity\PaymentSystem;
use XF\Mvc\Entity\Entity;
use XF\Mvc\ParameterBag;

class Requisites extends \XF\Api\Controller\AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertApiScopeByRequestMethod('pb_wq_requisite');
		$this->assertRegisteredUser();
	}

	/**
	 * @api-desc Gets the API user's list of withdrawal queue requisites.
	 *
	 * @api-out PB_WithdrawalQueue_Requisite[] $requisites
	 */
	public function actionGet()
	{
		$requisiteFinder = $this->setupRequisiteFinder();

		/** @var \PB\WithdrawalQueue\Entity\Requisite[]|\XF\Mvc\Entity\AbstractCollection $requisites */
		$requisites = $requisiteFinder->fetch();

		$results = $requisites->toApiResults();

		$return = [
			'requisites' => $results,
		];
		return $this->apiResult($return);
	}

	/**
	 * @return \XF\Finder\ConversationUser
	 */
	protected function setupRequisiteFinder()
	{
		return $this->getRequisiteRepo()->findRequisites()
			->forUser(\XF::visitor())
			->with('api');
	}

	/**
	 * @api-desc Creates the specified requisite
	 *
	 * @api-in int $system_id <req>
	 * @api-in str $wallet
	 * @api-in string $custom_fields[<name>]
	 *
	 * @api-out true $success
	 * @api-out PB_WithdrawalQueue_Requisite $requisite
	 */
	public function actionPost()
	{
		$this->assertRequiredApiInput(['system_id', 'wallet']);

		$paymentSystemId = $this->filter('system_id', 'uint');

		/** @var PaymentSystem $paymentSystem */
		$paymentSystem = $this->assertRecordExists(Addon::shortName('PaymentSystem'), $paymentSystemId);

		$editor = $this->setupRequisiteCreate($paymentSystem);
		if (!$editor->validate($errors))
		{
			return $this->error($errors);
		}

		$requisite = $editor->save();
		$this->finalizeRequisiteCreate($editor);

		return $this->apiSuccess([
			'requisite' => $requisite->toApiResult(Entity::VERBOSITY_VERBOSE)
		]);
	}

	protected function setupRequisiteCreate(PaymentSystem $paymentSystem)
	{
		/** @var \PB\WithdrawalQueue\Service\Requisite\Creator $creator */
		$creator = $this->service(Addon::shortName('Requisite\Creator'), $paymentSystem);
		$creator->setWallet($this->filter('wallet', 'str'));

		$customFields = $this->filter('custom_fields', 'array');
		$creator->setCustomFields($customFields);

		return $creator;
	}

	protected function finalizeRequisiteCreate(\PB\WithdrawalQueue\Service\Requisite\Creator $creator)
	{
	}

	/**
	 * @return \PB\WithdrawalQueue\Repository\Requisite|\XF\Mvc\Entity\Repository
	 */
	protected function getRequisiteRepo()
	{
		return $this->repository(Addon::shortName('Requisite'));
	}
}