<?php

namespace PB\WithdrawalQueue\Api\Controller;

use PB\WithdrawalQueue\Addon;
use PB\WithdrawalQueue\Entity\Withdraw as WithdrawEntity;
use XF\Mvc\Entity\Entity;
use XF\Mvc\ParameterBag;

class Withdraw extends \XF\Api\Controller\AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertApiScopeByRequestMethod('pb_wq_withdraw');
	}

	/**
	 * @api-desc Gets information about the specified withdraw.
	 *
	 * @api-out PB_WithdrawalQueue_Withdraw $withdraw
	 */
	public function actionGet(ParameterBag $params)
	{
		$withdraw = $this->assertWithdrawExists($params->withdraw_id);

		$result = [
			'withdraw' => $withdraw->toApiResult(Entity::VERBOSITY_VERBOSE)
		];

		return $this->apiResult($result);
	}

	protected function setupRequisiteEdit(WithdrawEntity $withdraw)
	{
		/** @var \PB\WithdrawalQueue\Service\Withdraw\Editor $editor */
		$editor = $this->service(Addon::shortName('Withdraw\Editor'), $withdraw);

		$input = $this->filter([
			'withdraw_state' => 'str',
			'payout' => 'float',
			'reject_reason' => 'str'
		]);

		if ($input['withdraw_state'])
		{
			$editor->setWithdrawState($input['withdraw_state']);
		}
		if ($input['payout'])
		{
			$editor->setPayout($input['payout']);
		}

		if ($input['withdraw_state'] == 'rejected')
		{
			$editor->setRejectReason($input['reject_reason']);
		}

		return $editor;
	}

	protected function finalizeWithdrawEdit(\PB\WithdrawalQueue\Service\Withdraw\Editor $editor)
	{
	}

	/**
	 * @api-desc Updates the specified withdraw
	 *
	 * @api-in str $withdraw_state Possible states: pending, sent, rejected
	 * @api-in float $payout
	 * @api-in str $reject_reason
	 *
	 * @api-out true $success
	 * @api-out PB_WithdrawalQueue_Withdraw $withdraw
	 */
	public function actionPost(ParameterBag $params)
	{
		$withdraw = $this->assertWithdrawExists($params->withdraw_id);

		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		if (\XF::isApiCheckingPermissions() && !$visitor->canModerateWithdraws($error))
		{
			return $this->noPermission($error);
		}

		$editor = $this->setupRequisiteEdit($withdraw);
		if (!$editor->validate($errors))
		{
			return $this->error($errors);
		}

		$editor->save();
		$this->finalizeWithdrawEdit($editor);

		return $this->apiSuccess([
			'withdraw' => $withdraw->toApiResult(Entity::VERBOSITY_VERBOSE)
		]);
	}

	/**
	 * @param int $id
	 * @param string|array $with
	 *
	 * @return WithdrawEntity|Entity
	 *
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertWithdrawExists($id, $with = 'api')
	{
		return $this->assertRecordExists(Addon::shortName('Withdraw'), $id, $with);
	}
}