<?php

namespace PB\WithdrawalQueue\Api\Controller;

use PB\WithdrawalQueue\Addon;
use XF\Mvc\ParameterBag;

class Withdraws extends \XF\Api\Controller\AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertApiScopeByRequestMethod('pb_wq_withdraw');
		$this->assertRegisteredUser();
	}

	/**
	 * @api-desc Gets the API user's list of withdrawal requests.
	 *
	 * @api-in int $page
	 * @api-in int $user_id
	 * @api-in string $withdraw_state Possible states: pending, sent, rejected
	 *
	 * @api-out PB_WithdrawalQueue_Withdraw[] $requisites
	 * @api-out pagination $pagination
	 */
	public function actionGet()
	{
		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		if (\XF::isApiCheckingPermissions() && !$visitor->canViewWithdraws($error))
		{
			return $this->error($error);
		}

		$page = $this->filterPage();
		$perPage = $this->options()->pb_wq_withdrawRop;

		$withdrawFinder = $this->setupWithdrawFinder();
		$withdrawFinder->limitByPage($page, $perPage);

		/** @var \PB\WithdrawalQueue\Entity\Withdraw[]|\XF\Mvc\Entity\AbstractCollection $withdraws */
		$withdraws = $withdrawFinder->fetch();
		$totalWithdraws = $withdrawFinder->total();

		$this->assertValidApiPage($page, $perPage, $totalWithdraws);

		$withdrawResults = $withdraws->toApiResults();

		$return = [
			'withdraws' => $withdrawResults,
			'pagination' => $this->getPaginationData($withdrawResults, $page, $perPage, $totalWithdraws)
		];
		return $this->apiResult($return);
	}

	/**
	 * @return \XF\Finder\ConversationUser
	 */
	protected function setupWithdrawFinder()
	{
		$withdrawFinder = $this->getWithdrawRepo()->findWithdraws();

		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		if (!$visitor->canModerateWithdraws())
		{
			$withdrawFinder->byUser($visitor);
		}
		else
		{
			$userId = $this->filter('user_id', 'uint');
			if ($userId)
			{
				$withdrawFinder->byUser($userId);
			}
		}

		$state = $this->filter('withdraw_state', 'str');
		if ($state)
		{
			$withdrawFinder->where('withdraw_state', '=', $state);
		}

		return $withdrawFinder->with('api');
	}

	/**
	 * @return \PB\WithdrawalQueue\Repository\Withdraw|\XF\Mvc\Entity\Repository
	 */
	protected function getWithdrawRepo()
	{
		return $this->repository(Addon::shortName('Withdraw'));
	}
}