<?php


namespace PB\WithdrawalQueue\CurrencyHandler;


use MJ\Credits\Util\Money;
use XF;
use XF\Entity\User;
use XF\Mvc\Entity\Entity;

class MJCredits extends AbstractHandler
{
	protected $currency;

	protected const DEFINITION_ID = 'pb_wq_withdraw';

	protected function getCurrency($currencyId)
	{
		if ($this->currency)
		{
			return $this->currency;
		}

		$plugin = $this->getCreditsPlugin();
		$currency = $plugin->useableCurrency(self::DEFINITION_ID, $currencyId);

		if ($currency)
		{
			$this->currency = $currency;
			return $currency;
		}

		return null;
	}

	public function getCurrencyTitle($currencyId)
	{
		$currency = $this->getCurrency($currencyId);
		if ($currency)
		{
			return $currency['title'];
		}

		return '';
	}

	public function getFormattedAmount($amount, $currencyId)
	{
		return Money::formatAmount($amount, $currencyId, true);
	}

	public function getChargedAmount($amount, $currencyId)
	{
		$creditsPlugin = $this->getCreditsPlugin();
		$currencies = $creditsPlugin->useableCurrencies(self::DEFINITION_ID);
		if (!$currencies)
		{
			return $amount;
		}

		$currency = $currencies[$currencyId];
		$event = $currency['event'];

		/** @var \MJ\Credits\Repository\Event $eventRepo */
		$eventRepo = XF::repository('MJ\Credits:Event');
		$fee = $eventRepo->calculateFee($amount, $event);

		return $amount - $fee;
	}

	public function getCurrencyOptionsData()
	{
		/** @var  \MJ\Credits\Repository\Currency $repo */
		$repo = XF::repository('MJ\Credits:Currency');
		$currencies = $repo->useableCurrencies('pb_wq_withdraw');

		$choices = [
			0 => ['_type' => 'option', 'value' => 0, 'label' => XF::phrase('(none)')]
		];

		foreach ($currencies AS $currencyId => $label)
		{
			$choices[$currencyId] = [
				'value' => $currencyId,
				'label' => $label['title']
			];
		}

		return array_map(function ($v) {
			$v['label'] = XF::escapeString($v['label']);
			return $v;
		}, $choices);
	}

	public function testCharge(Entity $content, User $user, $amount, $currencyId, &$errors = []): bool
	{
		$currency = $this->getCurrency($currencyId);

		if (!$currency)
		{
			$errors[] = XF::phrase('pb_wq_you_cannot_purchase_this');
			return false;
		}

		if ($user->get($currency['column']) < $amount)
		{
			$errors[] = XF::phrase('pb_wq_not_enough_x_to_purchase_this', [
				'amount' => Money::formatAmount($amount, $user->user_id)
			]);

			return false;
		}

		return true;
	}


	public function charge(Entity $content, User $user, $amount, $currencyId)
	{
		$currency = $this->getCurrency($currencyId);

		/** @var \MJ\Credits\Service\Event\Trigger $trigger */
		$trigger = XF::app()->service('MJ\Credits:Event\Trigger');

		$trigger->triggerEvent(self::DEFINITION_ID, $user->user_id, [
			'amount' => -$amount,
			'currency' => $currency,
			'event' => $currency['event']
		], $error);
	}

	public function testRefund(Entity $content, User $user, $amount, $currencyId, &$errors = []): bool
	{
		$currency = $this->getCurrency($currencyId);
		return !empty($currency);
	}

	public function refund(Entity $content, User $user, $amount, $currencyId)
	{
		$currency = $this->getCurrency($currencyId);

		/** @var \MJ\Credits\Service\Event\Trigger $trigger */
		$trigger = XF::app()->service('MJ\Credits:Event\Trigger');

		$trigger->triggerEvent(self::DEFINITION_ID, $user->user_id, [
			'amount' => $amount,
			'currency' => $currency,
			'event' => $currency['event']
		], $error);
	}

	/**
	 * @return \XF\ControllerPlugin\AbstractPlugin|\MJ\Credits\ControllerPlugin\Credit
	 */
	protected function getCreditsPlugin()
	{
		$app = XF::app();
		return $app->controller('XF:Index', $app->request())->plugin('MJ\Credits:Credit');
	}
}