<?php


namespace PB\WithdrawalQueue\Entity;


use Exception;
use LogicException;
use PB\WithdrawalQueue\CurrencyHandler\AbstractHandler;
use PB\WithdrawalQueue\Addon;
use XF;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * Class AbstractCurrency
 * @package PB\WithdrawalQueue\Entity
 *
 * COLUMNS
 * @property string $payout_currency
 * @property string $currency_addon
 * @property int $currency_id
 *
 * GETTERS
 * @property AbstractHandler $currency_handler
 * @property string $currency_title
 */
trait CurrencyHandlerTrait
{
	public function isCurrencyAddOnActive()
	{
		return XF::isAddOnActive($this->currency_addon);
	}

	public function getCurrencyTitle()
	{
		if ($this->currency_handler)
		{
			return $this->currency_handler->getCurrencyTitle($this->currency_id);
		}

		return '';
	}

	/**
	 * @return mixed|null|AbstractHandler
	 * @throws Exception
	 */
	public function getCurrencyHandler()
	{
		if (!$this->isCurrencyAddOnActive())
		{
			$currencyAddon = $this->app()->options()->pb_wq_currencyAddon;
		}
		else
		{
			$currencyAddon = $this->currency_addon;
		}

		/** @var \PB\WithdrawalQueue\Repository\Currency $currencyRepo */
		$currencyRepo = $this->repository(Addon::shortName('Currency'));
		$currencyAddons = $currencyRepo->getSupportedCurrencyAddons();
		$handlerClass = $currencyAddons[$currencyAddon]['handler_class'];

		$class = XF::stringToClass($handlerClass, '%s\CurrencyHandler\%s');
		if (!class_exists($class))
		{
			return null;
		}

		$class = XF::extendClass($class);
		return new $class();
	}

	protected static function addCurrencyHandlerStructureElements(Structure $structure)
	{
		$structure->columns += [
			'currency_addon' => ['type' => self::STR, 'required' => true, 'maxLength' => 50, 'api' => true],
			'currency_id' => ['type' => self::UINT, 'required' => true, 'api' => true],
			'payout_currency' => ['type' => self::STR, 'required' => true, 'maxLength' => 3, 'api' => true],
		];

		$structure->getters['currency_handler'] = true;
		$structure->getters['currency_title'] = true;
	}
}