<?php

namespace PB\WithdrawalQueue;

/**
 * Class Listener
 *
 * @package PB\WithdrawalQueue
 */
class Listener
{

	/**
	 * Called after the global \XF\App object has been setup. This will fire regardless of the
	 * application type.
	 *
	 * @param \XF\App $app Global App object.
	 * @throws \XF\Db\Exception
	 */
	public static function appSetup(\XF\App $app)
	{
		$container = $app->container();

		$container['pbWqPendingWithdrawCount'] = $app->fromRegistry(
			'pbWqPendingWithdrawCount',
			function (\XF\Container $c) {
				/** @var \PB\WithdrawalQueue\Repository\Withdraw $storeRepo */
				$storeRepo = $c['em']->getRepository(Addon::shortName('Withdraw'));
				return $storeRepo->rebuildPendingWithdrawCounts();
			}
		);

		$container['customFields.pb_wq_requisite'] = $app->fromRegistry('pbWqRequisiteFields',
			function (\XF\Container $c) {
				return $c['em']->getRepository(Addon::shortName('RequisiteField'))->rebuildFieldCache();
			},
			function (array $fields) use ($app) {
				$class = $app->extendClass('XF\CustomField\DefinitionSet');
				return new $class($fields);
			}
		);
	}


	/**
	 * Called at the end of the the Public \XF\Pub\App object startup process.
	 *
	 * @param \XF\Pub\App $app Public App object.
	 */
	public static function appPubStartEnd(\XF\Pub\App $app)
	{
		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		if ($visitor->canModerateWithdraws($error))
		{
			$session = $app->session();
			$registryWithdrawCounts = $app->container()->pbWqPendingWithdrawCount;
			$session->pbWqPendingWithdrawCount = $registryWithdrawCounts;
		}
	}
}