<?php

namespace PB\WithdrawalQueue\Option;

use PB\WithdrawalQueue\Addon;
use PB\WithdrawalQueue\Repository\Withdraw;
use XF;
use XF\Entity\Option;
use XF\Option\AbstractOption;

class CurrencyAddon extends AbstractOption
{
	public static function renderSelect(Option $option, array $htmlParams)
	{
		$data = self::getSelectData($option, $htmlParams);

		return self::getTemplater()->formSelectRow(
			$data['controlOptions'], $data['choices'], $data['rowOptions']
		);
	}

	public static function renderSelectMultiple(Option $option, array $htmlParams)
	{
		$data = self::getSelectData($option, $htmlParams);
		$data['controlOptions']['multiple'] = true;
		$data['controlOptions']['size'] = 8;

		return self::getTemplater()->formSelectRow(
			$data['controlOptions'], $data['choices'], $data['rowOptions']
		);
	}

	protected static function getSelectData(Option $option, array $htmlParams)
	{
		/** @var Withdraw $withdrawRepo */
		$withdrawRepo = XF::repository(Addon::shortName('Withdraw'));
		$currencyAddons = $withdrawRepo->getActiveCurrencyAddons();

		$choices = [
			0 => ['_type' => 'option', 'value' => 0, 'label' => XF::phrase('(none)')]
		];

		foreach ($currencyAddons AS $currencyAddonId => $label) {
			$choices[$currencyAddonId] = [
				'value' => $currencyAddonId,
				'label' => $label['title']
			];
		}

		$choices = array_map(function ($v) {
			$v['label'] = XF::escapeString($v['label']);
			return $v;
		}, $choices);

		return [
			'choices' => $choices,
			'controlOptions' => self::getControlOptions($option, $htmlParams),
			'rowOptions' => self::getRowOptions($option, $htmlParams)
		];
	}
}
