<?php


namespace PB\WithdrawalQueue\Pub\Controller;


use PB\WithdrawalQueue\Entity\Withdraw;
use PB\WithdrawalQueue\Addon;
use XF;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Repository;
use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\Exception;
use XF\Pub\Controller\AbstractController;

class Queue extends AbstractController
{
	public function actionIndex()
	{
		$withdrawRepo = $this->getWithdrawRepo();
		$finder = $withdrawRepo->findWithdraws()
			->with('Requisite')
			->onlyPending();

		$page = $this->filterPage();
		$perPage = 10;
		$finder->limitByPage($page, $perPage);
		$total = $finder->total();

		$this->assertValidPage($page, $perPage, $total, 'withdraw/queue');

		$redirect = $this->getDynamicRedirect($this->buildLink('withdraw/queue', null, ['page' => $page]), false);

		$viewParams = [
			'items' => $finder->fetch(),
			'total' => $total,
			'page' => $page,
			'perPage' => $perPage,
			'redirect' => $redirect
		];

		return $this->view(Addon::shortName('Queue\Listing'), Addon::prefix('queue_list'), $viewParams);
	}

	public function actionSave(ParameterBag $params)
	{
		$this->assertPostOnly();

		/** @var Withdraw $withdraw */
		$withdraw = $this->assertWithdrawExists($params->withdraw_id);

		$editor = $this->setupWithdrawEdit($withdraw);
		if (!$editor->validate($errors))
		{
			return $this->error($errors);
		}

		$editor->save();
		$this->finalizeWithdrawEdit($editor);

		$redirect = $this->getDynamicRedirect('withdraw/queue');
		return $this->redirect($redirect);
	}

	protected function setupWithdrawEdit(Withdraw $withdraw)
	{
		/** @var \PB\WithdrawalQueue\Service\Withdraw\Editor $editor */
		$editor = $this->service(Addon::shortName('Withdraw\Editor'), $withdraw);

		$input = $this->filter([
			'withdraw_state' => 'str',
			'payout' => 'float',
			'reason' => 'str'
		]);

		$editor->setWithdrawState($input['withdraw_state']);
		$editor->setPayout($input['payout']);

		if ($input['withdraw_state'] == 'rejected')
		{
			$editor->setRejectReason($input['reason']);
		}

		return $editor;
	}

	protected function finalizeWithdrawEdit(\PB\WithdrawalQueue\Service\Withdraw\Editor $editor)
	{
	}

	protected function preDispatchController($action, ParameterBag $params)
	{
		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = XF::visitor();
		if (!$visitor->canModerateWithdraws($error))
		{
			$reply = $this->noPermission($error);
			$reply->setPageParam('skipSidebarWidgets', true);

			throw $this->exception($reply);
		}

		parent::preDispatchController($action, $params);
	}

	public static function getActivityDetails(array $activities)
	{
		return Addon::phrase('viewing_withdraws');
	}

	/**
	 * @param $id
	 * @param null $with
	 * @param null $phraseKey
	 * @return Entity|\PB\WithdrawalQueue\Entity\Withdraw
	 * @throws Exception
	 */
	protected function assertWithdrawExists($id, $with = null, $phraseKey = 'pb_wq_requested_withdraw_not_found')
	{
		return $this->assertRecordExists(Addon::shortName('Withdraw'), $id, $with, $phraseKey);
	}

	/**
	 * @return Repository|\PB\WithdrawalQueue\Repository\Withdraw
	 */
	protected function getWithdrawRepo()
	{
		return $this->repository(Addon::shortName('Withdraw'));
	}
}