<?php


namespace PB\WithdrawalQueue\Pub\Controller;


use PB\WithdrawalQueue\Addon;
use XF;
use XF\App;
use XF\Http\Request;
use XF\Mvc\Entity\Repository;
use XF\Mvc\Reply\View;

class Withdraws extends AbstractBasicController
{
	public function __construct(App $app, Request $request)
	{
		parent::__construct($app, $request);

		$this->setFormatters(Addon::shortName('Withdraws%s'), Addon::prefix('withdraw_%s'));
		$this->setRoutePrefix('withdraw');
		$this->setEntityIdentifier(Addon::shortName('Withdraw'));
		$this->setPrimaryKey('withdraw_id');
	}

	protected function addWithdrawWrapperParams(View $view, $selected)
	{
		$view->setParam('pageSelected', $selected);
		return $view;
	}

	public function actionIndex()
	{
		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = XF::visitor();
		if (!$visitor->canViewWithdraws($error))
		{
			return $this->noPermission($error);
		}

		$withdrawRepo = $this->getWithdrawRepo();
		$finder = $withdrawRepo->findWithdraws()
			->with('PaymentSystem')
			->byUser($visitor);

		$page = $this->filterPage();
		$perPage = $this->options()->pb_wq_withdrawRop;
		$finder->limitByPage($page, $perPage);
		$total = $finder->total();

		$this->assertValidPage($page, $perPage, $total, $this->routePrefix);

		$viewParams = [
			'items' => $finder->fetch(),
			'total' => $total,
			'page' => $page,
			'link' => $this->routePrefix,
			'perPage' => $perPage,
		];

		$view = $this->view(
			$this->formatView('List'),
			$this->formatTemplate('list'),
			$viewParams
		);

		return $this->addWithdrawWrapperParams($view, 'withdraw');
	}

	public function actionRequisiteChoose()
	{
		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = XF::visitor();
		if (!$visitor->canCreateWithdraws($error))
		{
			return $this->noPermission();
		}

		/** @var \PB\WithdrawalQueue\Repository\Requisite $requisiteRepo */
		$requisiteRepo = $this->repository(Addon::shortName('Requisite'));
		$requisiteListData = $requisiteRepo->findUserRequisiteListData($visitor);
		if (empty($requisiteListData['systems']))
		{
			return $this->error(Addon::phrase('no_payment_systems_configured'));
		}

		$viewParams = [
			'link' => $this->routePrefix,
			'requisiteListData' => $requisiteListData,
		];

		$view = $this->view(
			$this->formatView('Add'),
			$this->formatTemplate('requisite_chooser'),
			$viewParams
		);

		return $this->addWithdrawWrapperParams($view, 'withdraw');
	}

	public static function getActivityDetails(array $activities)
	{
		return Addon::phrase('viewing_withdraws');
	}

	/**
	 * @return Repository|\PB\WithdrawalQueue\Repository\Withdraw
	 */
	protected function getWithdrawRepo()
	{
		return $this->repository(Addon::shortName('Withdraw'));
	}
}
