<?php


namespace PB\WithdrawalQueue\Repository;


use PB\WithdrawalQueue\Addon;
use XF\Entity\User;
use XF\Mvc\Entity\Finder;
use XF\Mvc\Entity\Repository;

class Requisite extends Repository
{
	public function findUserRequisiteListData(User $user)
	{
		$requisitesUngrouped = $this->findRequisites()->forUser($user)->fetch();

		/** @var PaymentSystem $paymentRepo */
		$paymentRepo = $this->repository(Addon::shortName('PaymentSystem'));
		$systems = $paymentRepo->findPaymentSystems()->onlyActive()->fetch();

		$uncategorized = [];
		$categoryKeys = $systems->keys();

		foreach ($requisitesUngrouped as $requisite)
		{
			if (!in_array($requisite['system_id'], $categoryKeys))
			{
				$uncategorized[] = $requisite;
			}
		}

		$requisites = $requisitesUngrouped->groupBy('system_id');
		$requisites = $this->em->getBasicCollection($requisites);
		$uncategorized = $this->em->getBasicCollection($uncategorized);

		return [
			'systems' => $systems,
			'uncategorized' => $uncategorized,
			'requisites' => $requisites,
			'requisitesUngrouped' => $requisitesUngrouped,
		];
	}

	/**
	 * @return Finder|\PB\WithdrawalQueue\Finder\Requisite
	 */
	public function findRequisites()
	{
		return $this->finder(Addon::shortName('Requisite'))
			->with('PaymentSystem', true)
			->setDefaultOrder('requisite_id');
	}

}