<?php


namespace PB\WithdrawalQueue\Repository;

use PB\WithdrawalQueue\Addon;
use XF;
use XF\Entity\User;
use XF\Mvc\Entity\Finder;
use XF\Mvc\Entity\Repository;

class Withdraw extends Repository
{
	/**
	 * @param null $state
	 * @return Finder|\PB\WithdrawalQueue\Finder\Withdraw
	 */
	public function findWithdraws()
	{
		$finder = $this->finder(Addon::shortName('Withdraw'));
		return $finder->setDefaultOrder('withdraw_id', 'DESC');
	}

	public function getActiveCurrencyAddons()
	{
		/** @var \PB\WithdrawalQueue\Repository\Currency $currencyRepo */
		$currencyRepo = $this->repository(Addon::shortName('Currency'));
		$currencyAddons = $currencyRepo->getSupportedCurrencyAddons();
		$addonCache = $this->app()->container('addon.cache');

		foreach ($currencyAddons as $addOnId => $addOn)
		{
			if (!isset($addonCache[$addOnId]))
			{
				unset($currencyAddons[$addOnId]);
			}
		}

		return $currencyAddons;
	}

	public function rebuildPendingWithdrawCounts()
	{
		$cache = [
			'total' => $this->db()->fetchOne("
				SELECT COUNT(*) FROM xf_pb_wq_withdraw
				WHERE `withdraw_state`='pending'
			"),
			'lastModified' => time()
		];

		\XF::registry()->set('pbWqPendingWithdrawCount', $cache);
		return $cache;
	}
}
