<?php

namespace PB\WithdrawalQueue\Service\Requisite;

use PB\WithdrawalQueue\Addon;
use XF\Service\ValidateAndSavableTrait;

class Creator extends \XF\Service\AbstractService
{
	use ValidateAndSavableTrait;

	/**
	 * @var \XF\Entity\User
	 */
	protected $user;

	/**
	 * @var \PB\WithdrawalQueue\Entity\Requisite
	 */
	protected $requisite;

	/**
	 * @var \PB\WithdrawalQueue\Entity\PaymentSystem
	 */
	protected $paymentSystem;


	protected $performValidations = true;

	public function __construct(\XF\App $app, \PB\WithdrawalQueue\Entity\PaymentSystem $paymentSystem)
	{
		parent::__construct($app);
		$this->paymentSystem = $paymentSystem;

		$this->setPaymentSystem($paymentSystem);
		$this->setupDefaults();
	}

	protected function setupDefaults()
	{
		$this->requisite = $this->paymentSystem->getNewRequisite();

		$user = \XF::visitor();
		$this->setUser($user);
	}

	public function setPaymentSystem(\PB\WithdrawalQueue\Entity\PaymentSystem $paymentSystem)
	{
		$this->paymentSystem = $paymentSystem;
	}

	public function setUser(\XF\Entity\User $user)
	{
		$this->user = $user;
		$this->requisite->user_id = $user->user_id;
	}

	public function setWallet(string $wallet)
	{
		$this->requisite->wallet = $wallet;
	}

	public function setCustomFields(array $customFields)
	{
		/** @var \XF\CustomField\Set $fieldSet */
		$fieldSet = $this->requisite->custom_fields;
		$fieldDefinition = $fieldSet->getDefinitionSet()
			->filterEditable($fieldSet, 'user')
			->filterOnly($this->paymentSystem->field_cache);

		$customFieldsShown = array_keys($fieldDefinition->getFieldDefinitions());

		if ($customFieldsShown)
		{
			$fieldSet->bulkSet($customFields, $customFieldsShown);
		}
	}

	public function setPerformValidations(bool $perform)
	{
		$this->performValidations = $perform;
	}

	protected function finalSetup()
	{
	}

	protected function _validate()
	{
		$this->requisite->preSave();
		$errors = $this->requisite->getErrors();

		if ($this->performValidations)
		{
			if (!$this->paymentSystem->active)
			{
				$errors[] = Addon::phrase('selected_payment_system_is_disabled');
			}

			if (!$this->requisite->validateWallet($error))
			{
				$errors[] = $error;
			}
		}

		$this->finalSetup();
		return $errors;
	}

	protected function afterInsert()
	{
	}

	protected function _save()
	{
		$requisite = $this->requisite;

		$requisite->save();
		$this->afterInsert();

		return $requisite;
	}
}