<?php

namespace PB\WithdrawalQueue\Service\Requisite;

use PB\WithdrawalQueue\Addon;
use XF\Service\ValidateAndSavableTrait;

class Editor extends \XF\Service\AbstractService
{
	use ValidateAndSavableTrait;

	/**
	 * @var \PB\WithdrawalQueue\Entity\Requisite
	 */
	protected $requisite;

	protected $performValidations = true;

	public function __construct(\XF\App $app, \PB\WithdrawalQueue\Entity\Requisite $requisite)
	{
		parent::__construct($app);

		$this->requisite = $requisite;
		$this->setupDefaults();
	}

	protected function setupDefaults()
	{
	}

	public function setWallet(string $wallet)
	{
		$this->requisite->wallet = $wallet;
	}

	public function setCustomFields(array $customFields)
	{
		/** @var \XF\CustomField\Set $fieldSet */
		$fieldSet = $this->requisite->custom_fields;
		$fieldDefinition = $fieldSet->getDefinitionSet()
			->filterEditable($fieldSet, 'user')
			->filterOnly($this->requisite->PaymentSystem->field_cache);

		$customFieldsShown = array_keys($fieldDefinition->getFieldDefinitions());

		if ($customFieldsShown)
		{
			$fieldSet->bulkSet($customFields, $customFieldsShown);
		}
	}

	public function setPerformValidations(bool $perform)
	{
		$this->performValidations = $perform;
	}

	protected function finalSetup()
	{
	}

	protected function _validate()
	{
		$this->requisite->preSave();
		$errors = $this->requisite->getErrors();

		if ($this->performValidations)
		{
			if (!$this->requisite->PaymentSystem || !$this->requisite->PaymentSystem->active)
			{
				$errors[] = Addon::phrase('selected_payment_system_is_disabled');
			}

			if (!$this->requisite->validateWallet($error))
			{
				$errors[] = $error;
			}
		}

		$this->finalSetup();
		return $errors;
	}

	protected function afterUpdate()
	{
	}

	protected function _save()
	{
		$requisite = $this->requisite;

		$requisite->save();
		$this->afterUpdate();

		return $requisite;
	}
}