<?php

namespace PB\WithdrawalQueue\Service\Withdraw;

use PB\WithdrawalQueue\Addon;
use PB\WithdrawalQueue\CurrencyHandler\AbstractHandler;
use PB\WithdrawalQueue\Entity\Requisite;
use PB\WithdrawalQueue\Entity\Withdraw;
use XF\Service\ValidateAndSavableTrait;

class Creator extends \XF\Service\AbstractService
{
	use ValidateAndSavableTrait;

	/**
	 * @var \XF\Entity\User
	 */
	protected $user;

	/**
	 * @var Requisite
	 */
	protected $requisite;

	/**
	 * @var Withdraw
	 */
	protected $withdraw;

	protected $performValidations = true;

	public function __construct(\XF\App $app, Requisite $requisite)
	{
		parent::__construct($app);

		$this->withdraw = $this->em()->create(Addon::shortName('Withdraw'));

		$this->setRequisite($requisite);
		$this->setupDefaults();
	}

	protected function setupDefaults()
	{
		$user = \XF::visitor();
		$this->setUser($user);

		$paymentSystem = $this->requisite->PaymentSystem;

		$this->withdraw->payout_currency = $paymentSystem->payout_currency;
		$this->withdraw->currency_addon = $paymentSystem->currency_addon;
		$this->withdraw->currency_id = $paymentSystem->currency_id;
	}

	public function setRequisite(Requisite $requisite)
	{
		$this->requisite = $requisite;
		$this->withdraw->system_id = $requisite->system_id;
		$this->withdraw->system_title = $requisite->PaymentSystem->title;
		$this->withdraw->requisite_id = $requisite->requisite_id;
		$this->withdraw->wallet = $requisite->wallet;
	}

	public function setUser(\XF\Entity\User $user)
	{
		$this->user = $user;
		$this->withdraw->user_id = $user->user_id;
		$this->withdraw->username = $user->username;
	}

	public function setAmount(float $amount)
	{
		$this->withdraw->sum = $amount;
		$this->withdraw->payout = $this->requisite->PaymentSystem->getActualPayoutAmount($amount);
	}

	public function setPerformValidations(bool $perform)
	{
		$this->performValidations = $perform;
	}

	protected function finalSetup()
	{
		$this->withdraw->create_date = \XF::$time;
	}

	protected function _validate()
	{
		$this->finalSetup();

		$withdraw = $this->withdraw;
		$withdraw->preSave();
		$errors = $withdraw->getErrors();

		if ($this->performValidations)
		{
			$paymentSystem = $this->requisite->PaymentSystem;
			if (!$paymentSystem || !$paymentSystem->active)
			{
				$errors[] = Addon::phrase('selected_payment_system_is_disabled');
			}

			$minSum = $paymentSystem->min_amount;
			if ($withdraw->sum < $minSum)
			{
				$errors[] = \XF::phrase('please_enter_number_that_is_at_least_x', ['min' => $minSum]);
			}

			if ($withdraw->payout <= 0)
			{
				$errors[] = Addon::phrase('payout_sum_must_be_positive');
			}
		}

		if ($errors)
		{
			return $errors;
		}

		if (!$withdraw->testCharge($errors) && !$errors)
		{
			$errors[] = \XF::phrase('pb_wq_failed_to_charge_funds');
		}

		return $errors;
	}

	protected function afterInsert()
	{
	}

	protected function _save()
	{
		$withdraw = $this->withdraw;

		$db = $this->db();
		$db->beginTransaction();

		$withdraw->save();
		$withdraw->charge();

		$db->commit();

		$this->afterInsert();

		return $withdraw;
	}
}