<?php

namespace PB\WithdrawalQueue\Service\Withdraw;

use PB\WithdrawalQueue\Addon;
use PB\WithdrawalQueue\Entity\Withdraw;
use XF\Service\ValidateAndSavableTrait;

class Editor extends \XF\Service\AbstractService
{
	use ValidateAndSavableTrait;

	/**
	 * @var Withdraw
	 */
	protected $withdraw;

	protected $performValidations = true;

	protected $newState;

	public function __construct(\XF\App $app, Withdraw $withdraw)
	{
		parent::__construct($app);

		$this->withdraw = $withdraw;
	}

	public function setPayout(float $payout)
	{
		$this->withdraw->payout = $payout;
	}

	public function setWithdrawState($state)
	{
		$this->withdraw->withdraw_state = $state;
	}

	public function setPerformValidations($perform = true)
	{
		$this->performValidations = $perform;
	}

	public function setRejectReason($reason)
	{
		$this->withdraw->reject_reason = $reason;
	}

	protected function finalSetup()
	{
	}

	protected function afterUpdate()
	{
	}

	protected function onApprove()
	{
		/** @var Notifier $notifier */
		$notifier = $this->service(Addon::shortName('Withdraw\Notifier'), $this->withdraw);
		$notifier->addNotification('complete', $this->withdraw->user_id, true, false);
		$notifier->notify();
	}

	protected function onReject()
	{
		$withdraw = $this->withdraw;

		if ($withdraw->testRefund())
		{
			$withdraw->refund();
		}

		/** @var Notifier $notifier */
		$notifier = $this->service(Addon::shortName('Withdraw\Notifier'), $this->withdraw);
		$notifier->addNotification('reject', $this->withdraw->user_id, true, false);
		$notifier->notify();
	}

	protected function _validate()
	{
		$this->finalSetup();
		$errors = [];

		$withdraw = $this->withdraw;
		if (!$withdraw->preSave())
		{
			$errors = $withdraw->getErrors();
		}

		if ($this->performValidations)
		{
			if ($this->withdraw->payout <= 0)
			{
				$errors[] = \XF::phrase('please_enter_number_that_is_at_least_x', ['min' => 0.01]);
			}
		}

		return $errors;
	}

	protected function _save()
	{
		$withdraw = $this->withdraw;

		$db = $this->db();
		$db->beginTransaction();

		if ($withdraw->isStateChanged('withdraw_state', 'rejected') == 'enter')
		{
			$this->onReject();
		}
		elseif ($withdraw->isStateChanged('withdraw_state', 'sent') == 'enter')
		{
			$this->onApprove();
		}

		$withdraw->save(true, false);
		$this->afterUpdate();

		$db->commit();

		return $withdraw;
	}
}
