<?php

namespace PB\WithdrawalQueue;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;

	// ################################## INSTALL ###########################################

	public function installStep1()
	{
		$sm = $this->schemaManager();

		foreach ($this->getTables() as $tableName => $closure)
		{
			$sm->createTable($tableName, $closure);
		}
	}

	// ################################## UNINSTALL ###########################################

	public function uninstallStep1()
	{
		$sm = $this->schemaManager();

		foreach (array_keys($this->getTables()) as $tableName)
		{
			$sm->dropTable($tableName);
		}
	}

	// ################################## UPGRADE ###########################################

	// 2.1.0

	public function upgrade2010000Step1()
	{
		$this->alterTable('xf_pb_wq_withdraw', function (Alter $table) {
			$table->changeColumn('sum', 'decimal', '19,8');
			$table->changeColumn('payout', 'decimal', '19,8');
			$table->addColumn('charged', 'decimal', '19,8')->after('payout')->nullable()->setDefault(null);
			$table->addColumn('requisite_id', 'int')->after('withdraw_state')->setDefault(0);
		});

		$this->alterTable('xf_pb_wq_payment_system', function (Alter $table) {
			$table->changeColumn('tax', 'decimal', '19,8')->setDefault(0);
			$table->changeColumn('percent', 'decimal', '19,8')->setDefault(0);

			$table->addColumn('input_type', 'varbinary', '50')->after('description')->setDefault('text');
			$table->addColumn('min_amount', 'decimal', '19,8')->after('percent')->setDefault(0);
			$table->addColumn('step_amount', 'decimal', '19,8')->after('percent')->setDefault(0);
			$table->addColumn('decimal_places', 'tinyint')->after('percent')->setDefault(0);
			$table->addColumn('currency_rate', 'decimal', '19,8')->after('currency_id')->setDefault(1);
			$table->addColumn('field_cache', 'mediumblob');
		});

		$this->alterTable('xf_pb_wq_requisite', function (Alter $table) {
			$table->addColumn('custom_fields', 'mediumblob');
		});

		$this->createTable('xf_pb_wq_requisite_field', function (Create $table) {
			$table->addColumn('field_id', 'varbinary', 25);
			$table->addColumn('display_group', 'varchar', 25)->setDefault('above_info');
			$table->addColumn('display_order', 'int')->setDefault(1);
			$table->addColumn('field_type', 'varbinary', 25)->setDefault('textbox');
			$table->addColumn('field_choices', 'blob');
			$table->addColumn('match_type', 'varbinary', 25)->setDefault('none');
			$table->addColumn('match_params', 'blob');
			$table->addColumn('max_length', 'int')->setDefault(0);
			$table->addColumn('required', 'tinyint')->setDefault(0);
			$table->addColumn('display_template', 'text');
			$table->addColumn('wrapper_template', 'text');
			$table->addPrimaryKey('field_id');
			$table->addKey(['display_group', 'display_order'], 'display_group_order');
		});

		$this->createTable('xf_pb_wq_requisite_field_value', function (Create $table) {
			$table->addColumn('requisite_id', 'int');
			$table->addColumn('field_id', 'varbinary', 25);
			$table->addColumn('field_value', 'mediumtext');
			$table->addPrimaryKey(['requisite_id', 'field_id']);
			$table->addKey('field_id');
		});

		$this->createTable('xf_pb_wq_payment_system_field', function (Create $table) {
			$table->addColumn('system_id', 'int');
			$table->addColumn('field_id', 'varbinary', 25);
			$table->addPrimaryKey(['system_id', 'field_id']);
			$table->addKey('field_id');
		});
	}

	public function upgrade2010000Step2()
	{
		/** @var \PB\WithdrawalQueue\Entity\PaymentSystem[] $paymentSystems */
		$paymentSystems = $this->app->finder(Addon::shortName('PaymentSystem'));

		foreach ($paymentSystems as $paymentSystem)
		{
			$updates = [];

			$inputType = $paymentSystem->getExtraDataValue('input_type');
			if ($inputType)
			{
				$updates['input_type'] = $inputType;
			}

			$minSum = $paymentSystem->getExtraDataValue('min_sum');
			if ($inputType)
			{
				$updates['min_amount'] = $minSum;
			}

			$paymentSystem->fastUpdate($updates);
		}
	}

	// 2.1.0 Alpha 1

	public function upgrade2010011Step1()
	{
		$this->alterTable('xf_pb_wq_withdraw', function (Alter $table) {
			$table->addColumn('reject_reason', 'varchar', 255)->setDefault('')->after('withdraw_state');
		});
	}

	// 2.1.1

	public function upgrade2010100Step1()
	{
		$this->alterTable('xf_pb_wq_payment_system', function (Alter $table) {
			$table->addColumn('image_date', 'int')->setDefault(0)->after('decimal_places');
		});
	}

	// ################################## DATA ###########################################

	/**
	 * @return array
	 */
	protected function getTables(): array
	{
		$tables = [];

		$tables['xf_pb_wq_withdraw'] = function (Create $table) {
			$table->addColumn('withdraw_id', 'int')->autoIncrement();
			$table->addColumn('user_id', 'int');
			$table->addColumn('username', 'varchar', 50);
			$table->addColumn('update_user_id', 'int')->setDefault(0);
			$table->addColumn('update_username', 'varchar', 50)->setDefault('');

			$table->addColumn('currency_addon', 'varbinary', 50);
			$table->addColumn('currency_id', 'int');

			$table->addColumn('sum', 'decimal', '19,8');
			$table->addColumn('payout', 'decimal', '19,8');
			$table->addColumn('charged', 'decimal', '19,8')->nullable()->setDefault(null);

			$table->addColumn('payout_currency', 'varchar', 3);

			$table->addColumn('requisite_id', 'int')->setDefault(0);
			$table->addColumn('wallet', 'varchar', 255);

			$table->addColumn('withdraw_state', 'enum')->values(['pending', 'rejected', 'sent'])->setDefault('pending');
			$table->addColumn('reject_reason', 'varchar', 255)->setDefault('');

			$table->addColumn('system_id', 'int');
			$table->addColumn('system_title', 'varchar', 255);

			$table->addColumn('create_date', 'int')->setDefault(0);
			$table->addColumn('update_date', 'int')->setDefault(0);

			$table->addPrimaryKey('withdraw_id');
			$table->addKey('user_id');
			$table->addKey('update_user_id');
		};

		$tables['xf_pb_wq_requisite'] = function (Create $table) {
			$table->addColumn('requisite_id', 'int')->autoIncrement();
			$table->addColumn('system_id', 'int');
			$table->addColumn('user_id', 'int');

			$table->addColumn('wallet', 'varchar', 255);

			$table->addColumn('create_date', 'int')->setDefault(0);
			$table->addColumn('update_date', 'int')->setDefault(0);

			$table->addColumn('custom_fields', 'mediumblob');

			$table->addPrimaryKey('requisite_id');
			$table->addKey('system_id');
			$table->addKey('user_id');
			$table->addUniqueKey('wallet');
		};

		$tables['xf_pb_wq_requisite_field'] = function(Create $table)
		{
			$table->addColumn('field_id', 'varbinary', 25);
			$table->addColumn('display_group', 'varchar', 25)->setDefault('above_info');
			$table->addColumn('display_order', 'int')->setDefault(1);
			$table->addColumn('field_type', 'varbinary', 25)->setDefault('textbox');
			$table->addColumn('field_choices', 'blob');
			$table->addColumn('match_type', 'varbinary', 25)->setDefault('none');
			$table->addColumn('match_params', 'blob');
			$table->addColumn('max_length', 'int')->setDefault(0);
			$table->addColumn('required', 'tinyint')->setDefault(0);
			$table->addColumn('display_template', 'text');
			$table->addColumn('wrapper_template', 'text');
			$table->addPrimaryKey('field_id');
			$table->addKey(['display_group', 'display_order'], 'display_group_order');
		};

		$tables['xf_pb_wq_requisite_field_value'] = function(Create $table)
		{
			$table->addColumn('requisite_id', 'int');
			$table->addColumn('field_id', 'varbinary', 25);
			$table->addColumn('field_value', 'mediumtext');
			$table->addPrimaryKey(['requisite_id', 'field_id']);
			$table->addKey('field_id');
		};

		$tables['xf_pb_wq_payment_system'] = function (Create $table) {
			$table->addColumn('system_id', 'int')->autoIncrement();
			$table->addColumn('title', 'varchar', 255);
			$table->addColumn('description', 'text');
			$table->addColumn('input_type', 'varbinary', '50')->setDefault('text');

			$table->addColumn('currency_addon', 'varbinary', 50);
			$table->addColumn('currency_id', 'int');
			$table->addColumn('currency_rate', 'decimal', '19,8')->setDefault(1);

			$table->addColumn('payout_currency', 'varchar', 3);

			$table->addColumn('tax', 'decimal', '19,8')->setDefault(0);
			$table->addColumn('percent', 'decimal', '19,8')->setDefault(0);
			$table->addColumn('min_amount', 'decimal', '19,8')->setDefault(0);
			$table->addColumn('step_amount', 'decimal', '19,8')->setDefault(0);
			$table->addColumn('decimal_places', 'tinyint')->setDefault(0);

			$table->addColumn('image_date', 'int')->setDefault(0);
			$table->addColumn('display_order', 'int')->setDefault(0);
			$table->addColumn('active', 'tinyint')->setDefault(1);

			$table->addColumn('extra_data', 'blob')->nullable(true);
			$table->addColumn('field_cache', 'mediumblob');

			$table->addPrimaryKey('system_id');
			$table->addKey('system_id');
			$table->addKey('active');
		};

		$tables['xf_pb_wq_payment_system_field'] = function(Create $table)
		{
			$table->addColumn('system_id', 'int');
			$table->addColumn('field_id', 'varbinary', 25);
			$table->addPrimaryKey(['system_id', 'field_id']);
			$table->addKey('field_id');
		};

		return $tables;
	}
}
